#include <stdio.h>
#include <string.h>

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "lcd_lib.h"

#include "ff.h"
#include "diskio.h"

//   Old testing functions
			
void DisplayRout()
{
	static int i = 0;
	static char aa = 'a';

	LCDsendChar(aa++);

	if (i == 16)
		LCDGotoXY(0,1);

	if (i == 32)
	{
		LCDGotoXY(0,0);
		LCDclr();  
		i = 0;
	}

	i++;		
}

#define  TIMER unsigned char

TIMER _t_sd_timer = 0;
TIMER _t_lcd_timer = 0;

ISR (TIMER2_COMP_vect)		//  1 ms
{
	if (_t_sd_timer)
		--_t_sd_timer;
	else
	{
		_t_sd_timer = 11;
		disk_timerproc();
	}

	if (_t_lcd_timer)
	  _t_lcd_timer--;
}


static void IOInit ()
{
	OCR2 = (117-1)/10;										//  Timer2: 1000 Hz (1 ms) interval (OC2)
	TCCR2 = (1<<WGM21)|(1<<CS22)|(1<<CS20);				//  xtal / 1024

	TIMSK = (1<<OCIE2);	// Enable TC2.oc, interrupt
	
	sei(); 
}



FRESULT scan_files (char* path)		//Scan files and directories on path
{
	FRESULT res;
	FILINFO fno;
	DIR dir;
	int i;
	char *fn;

    res = f_opendir(&dir, path);

    if (res == FR_OK)
	{
		i = strlen(path);
		for (;;) 
		{
			res = f_readdir(&dir, &fno);		//Read dir items

			if (res != FR_OK || fno.fname[0] == 0)
				break;

			if (fno.fname[0] == '.')
				continue;
			
			fn = fno.fname;
			
			if (fno.fattrib & AM_DIR)
			{		// Check item type, DIR or FILE
				PrintStr("DIR: ");
				PrintStr (fn);
			} 
			else
			{
				//PrintStr ("FILE: ");
			    PrintStr (fn);
			}
		}
	}

    return res;
}


void _proc_LCD()
{	
	static char step = 0;

	if (step == 0)
	{
		if ( LCDinit() )
			step=3;
	}

	if (step == 1)
	{
	//	if (LCDcursorOFF()) //   
	//		step++;
	}

	if (step == 2)
	{
	//	if (LCDclr())  			//   
	//		step++;
	}

	if (step == 3)
	{
		PrintStr ("multiproc success");		
		step++;
	}
}


int main( void )
{	 
	IOInit ();					//  Init Timer2


	LCDinit();
	LCDcursorOFF(); //   
	LCDclr();  			//   


	FRESULT f_err_code;
	static FATFS FATFS_Obj;
	FIL fil_obj;

	disk_initialize(0);		//  Init drive	
	f_err_code = f_mount(0, &FATFS_Obj);	//  Mount Fat Fs

	PrintStr ("mounting FAT ");

	f_err_code = f_open(&fil_obj, "newfile.txt", FA_READ);	  //  Open newfile for reading

	PrintStr ("open newfile.txt");

	if (f_err_code)
	{
		PrintStr("FAIL");
	}
	else
	{
		PrintStr ("OK");

		PrintStr ("data in newfile.txt:");	
		char buff[12] = {0};
		f_gets (buff, 12, &fil_obj);	//  Read data from newfile

		f_err_code = f_close(&fil_obj);

		PrintStr (buff);	
	}


	f_err_code = f_open(&fil_obj, "new.txt",FA_CREATE_NEW | FA_WRITE);	//  Create newfile into newdir
	PrintStr ("creating newfile ");
		
	if(!f_err_code)
		PrintStr ("OK");
	else
		PrintStr("FAIL");

	f_puts ("creating nad adnandnad", &fil_obj);		//  Writing to newfile

	f_err_code = f_close(&fil_obj);		//  Close newfile
	PrintStr ("closing newfile.txt ");


	PrintStr("scaning newdir:");
	scan_files ("0:/");							//  Scan newdir
	PrintStr("scaning ended");

									
	f_err_code = f_mount(0, NULL);			//  Unmount Fat Fs
	PrintStr ("unmounting FAT ");

	while(1)
	{
	//	_delay_ms(300);
	//	DisplayRout();
//		_proc_LCD();
	}
}
