//*****************************************************************************
//
// File Name	: 'lcd_lib.c'
// Title		: 4 bit LCd interface
// Author		: Scienceprog.com - Copyright (C) 2007
// Modified	by	: Koryagin Andrey 2011 modified by Anton chepeluck 2013
// Created		: 2007-06-18
// Modified		: 2013-05-20
// Version		: 1.1
// Target MCU	: Atmel AVR series
//
// This code is distributed under the GNU Public License
//		which can be found at http://www.gnu.org/licenses/gpl.txt
//
//*****************************************************************************
#include "lcd_lib.h"
#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>


char LCD_str1[LCD_STR_SIZE] = {0};
char LCD_str2[LCD_STR_SIZE] = {0};


#ifdef __ASYNC_MOD__

#define TIMER unsigned char
extern TIMER _t_lcd_timer;

char SetLCDTimer (TIMER timer, unsigned char *lcd_step)
{
	_t_lcd_timer = timer;
	(*lcd_step)++;

	return 0;
}

char LCDCheckNewStep (unsigned char *lcd_step)
{
	if (!_t_lcd_timer)
		(*lcd_step)++;

	return 0;
}

char LCDsendChar(uint8_t ch)		//  Sends Char to LCD
{
	LCP |=  1<<LCD_A0; 
	LCP |=  1<<LCD_E;		

//	LDP = ch;
	LDP &= ~((0b11110000)>>(4-LCD_D4));
	LDP |=  ((ch&0b11110000)>>(4-LCD_D4));

	_delay_ms(1);
	LCP &= ~(1<<LCD_E);
		
	_delay_ms(1);

	LCP |=  1<<LCD_E;		
//	LDP = ch << 4;
	LDP &= ~((0b11110000)>>(4-LCD_D4));
	LDP |=  ((ch&0b00001111)<<(LCD_D4));	

	_delay_ms(1);	
	LCP &= ~(1<<LCD_E);	

	_delay_ms(1);
	LCP &= ~(1<<LCD_A0);

	return 0;
}

char LCDsendCommand(uint8_t cmd)
{
	LDP &= ~((0b11110000)>>(4-LCD_D4));
	LDP |=((cmd&0b11110000)>>(4-LCD_D4));

	LCP|=1<<LCD_E;		
	_delay_ms(1);
	LCP&=~(1<<LCD_E);
	_delay_ms(1);

	LDP &= ~((0b11110000)>>(4-LCD_D4));
	LDP |=((cmd&0b00001111)<<(LCD_D4));

	LCP|=1<<LCD_E;		
	_delay_ms(1);
	LCP&=~(1<<LCD_E);
	_delay_ms(1);	

	return 0;
}

char _LCDsendCommand(uint8_t cmd)
{
	static unsigned char lcd_step = 0;

	if (lcd_step & 0x1)
		return LCDCheckNewStep(&lcd_step);

	if (lcd_step == 0)
	{		
		LDP &= ~((0b11110000)>>(4-LCD_D4));
		LDP |=((cmd&0b11110000)>>(4-LCD_D4));
		
		LCP |=1<<LCD_E;		

		return SetLCDTimer(1, &lcd_step);
	}

	if (lcd_step == 2)
	{
		LCP &= ~(1<<LCD_E);
		return SetLCDTimer(1, &lcd_step);
	}

	if (lcd_step == 4)
	{

		LDP &= ~((0b11110000)>>(4-LCD_D4));
		LDP |=((cmd&0b00001111)<<(LCD_D4));

		LCP |=1<<LCD_E;		

		return SetLCDTimer(1, &lcd_step);
	}

	if (lcd_step == 6)
	{
		LCP &= ~(1<<LCD_E);

		return SetLCDTimer(1, &lcd_step);
	}

	return 1;	
}

char LCDinit()  //  Initializes LCD
{
	static unsigned char lcd_step = 0;

	if (lcd_step & 0x1)
		return LCDCheckNewStep(&lcd_step);

	if (lcd_step == 0)
	{		
		return SetLCDTimer(100, &lcd_step);
	}

	if (lcd_step == 2)
	{
		LDP &= ~(1<<LCD_D7|1<<LCD_D6|1<<LCD_D5|1<<LCD_D4);
		LCP &= ~(1<<LCD_E);  //  |1<<LCD_RS);
		LDDR|=1<<LCD_D7|1<<LCD_D6|1<<LCD_D5|1<<LCD_D4;
		LCDR|=1<<LCD_E|1<<LCD_A0|1<<LCD_RW; 

		LCP &=  ~(1<<LCD_A0); 
		LCP &=  ~(1<<LCD_RW); 
   		
		//---------one------
		//4 bit mode
		LDP |= (1<<LCD_D5|1<<LCD_D4);
		LDP &= ~(1<<LCD_D7|1<<LCD_D6);
		LCP |= (1<<LCD_E);

		return SetLCDTimer(20, &lcd_step);
	}
	
	if (lcd_step == 4)
	{
		LCP &= ~(1<<LCD_E);
		return SetLCDTimer(20, &lcd_step);
	}

	if (lcd_step == 6)
	{
		//-----------two-----------
		LDP |= (1<<LCD_D5|1<<LCD_D4);
		LDP &= ~(1<<LCD_D7|1<<LCD_D6);
		LCP |= (1<<LCD_E);

		return SetLCDTimer(20, &lcd_step);
	}
			
	if (lcd_step == 8)
	{
		LCP &= ~(1<<LCD_E);
		return SetLCDTimer(20, &lcd_step);
	}
		
	if (lcd_step == 10)
	{
		//-------three-------------
		LDP |= (1<<LCD_D5);
		LDP &= ~(1<<LCD_D7|1<<LCD_D6|1<<LCD_D4);
		LCP|= (1<<LCD_E);
		return SetLCDTimer(20, &lcd_step);
	}

	if (lcd_step == 12)
	{	
		LCP &= ~(1<<LCD_E);
		return SetLCDTimer(20, &lcd_step);
	}

	if (lcd_step == 14)
	{
		//--------4 bit--dual line---------------
		if (_LCDsendCommand(0b00101000))		
			lcd_step++;
		
		return 0;
	}

	if (lcd_step == 16)
	{
	   //-----increment address, cursor shift------
		if (_LCDsendCommand(0b00001110))
			lcd_step++;

		return 0;
	}

	return 1;
}

#endif  // __ASYNC_MOD__

//===================== NO ASYCNC MODE =====================================//

#ifndef __ASYNC_MOD__

char LCDsendChar(uint8_t ch)		//  Sends Char to LCD
{
	LCP |=  1<<LCD_A0; 
	LCP |=  1<<LCD_E;		

//	LDP = ch;
	LDP &= ~((0b11110000)>>(4-LCD_D4));
	LDP |=  ((ch&0b11110000)>>(4-LCD_D4));

	_delay_ms(1);
	LCP &= ~(1<<LCD_E);
		
	_delay_ms(1);

	LCP |=  1<<LCD_E;		
//	LDP = ch << 4;
	LDP &= ~((0b11110000)>>(4-LCD_D4));
	LDP |=  ((ch&0b00001111)<<(LCD_D4));	

	_delay_ms(1);	
	LCP &= ~(1<<LCD_E);	

	_delay_ms(1);
	LCP &= ~(1<<LCD_A0);

	return 0;
}

char LCDsendCommand(uint8_t cmd)
{
	LDP &= ~((0b11110000)>>(4-LCD_D4));
	LDP |=((cmd&0b11110000)>>(4-LCD_D4));

	LCP|=1<<LCD_E;		
	_delay_ms(1);
	LCP&=~(1<<LCD_E);
	_delay_ms(1);

	LDP &= ~((0b11110000)>>(4-LCD_D4));
	LDP |=((cmd&0b00001111)<<(LCD_D4));

	LCP|=1<<LCD_E;		
	_delay_ms(1);
	LCP&=~(1<<LCD_E);
	_delay_ms(1);	

	return 0;
}


char LCDinit(void)//Initializes LCD
{
	_delay_ms(100);

	LDP &= ~(1<<LCD_D7|1<<LCD_D6|1<<LCD_D5|1<<LCD_D4);
	LCP &= ~(1<<LCD_E);//|1<<LCD_RS);
	LDDR|=1<<LCD_D7|1<<LCD_D6|1<<LCD_D5|1<<LCD_D4;
	LCDR|=1<<LCD_E|1<<LCD_A0|1<<LCD_RW; 

	LCP &=  ~(1<<LCD_A0); 
	LCP &=  ~(1<<LCD_RW); 
   //---------one------
	//4 bit mode
	LDP |= (1<<LCD_D5|1<<LCD_D4);
	LDP &= ~(1<<LCD_D7|1<<LCD_D6);
	LCP|= (1<<LCD_E);
	_delay_ms(20);
	LCP &= ~(1<<LCD_E);
	_delay_ms(20);
	//-----------two-----------
	LDP |= (1<<LCD_D5|1<<LCD_D4);
	LDP &= ~(1<<LCD_D7|1<<LCD_D6);
	LCP|= (1<<LCD_E);
	_delay_ms(20);
	LCP &= ~(1<<LCD_E);
	_delay_ms(20);

	//-------three-------------
	LDP |= (1<<LCD_D5);
	LDP &= ~(1<<LCD_D7|1<<LCD_D6|1<<LCD_D4);
	LCP|= (1<<LCD_E);
	_delay_ms(20);
	LCP &= ~(1<<LCD_E);
	_delay_ms(20);

	//--------4 bit--dual line---------------
	LCDsendCommand(0b00101000);			//  English
	//	LCDsendCommand(0b00101010);			//  Russian

   //-----increment address, cursor shift------
	LCDsendCommand(0b00001110);

	return 0;
}

#endif  // __ASYNC_MOD__

char LCDclr(void)				//  Clears LCD
{
	return LCDsendCommand(1<<LCD_CLR);
	
}

char LCDhome(void)			//  LCD cursor home
{
	return LCDsendCommand(1<<LCD_HOME);
}

void LCDstring(uint8_t* data, uint8_t nBytes)	//Outputs string to LCD
{
	register uint8_t i;

	// check to make sure we have a good pointer
	if (!data)
	  return;

	// print data
	for (i = 0; i < nBytes; i++)
		LCDsendChar(data[i]);
}

char LCDGotoXY(uint8_t x, uint8_t y)	//Cursor to X Y position
{
	register uint8_t DDRAMAddr;
	// remap lines into proper order
	switch(y)
	{
		case 0: DDRAMAddr = LCD_LINE0_DDRAMADDR+x; break;
		case 1: DDRAMAddr = LCD_LINE1_DDRAMADDR+x; break;
		case 2: DDRAMAddr = LCD_LINE2_DDRAMADDR+x; break;
		case 3: DDRAMAddr = LCD_LINE3_DDRAMADDR+x; break;
		default: DDRAMAddr = LCD_LINE0_DDRAMADDR+x;
	}
	// set data address
	return LCDsendCommand(1<<LCD_DDRAM | DDRAMAddr);	
}

//Copies string from flash memory to LCD at x y position
//const uint8_t welcomeln1[] PROGMEM="AVR LCD DEMO\0";
//CopyStringtoLCD(welcomeln1, 3, 1);	
void CopyStringtoLCD(const uint8_t *FlashLoc, uint8_t x, uint8_t y)
{
	uint8_t i;
	
	LCDGotoXY(x,y);
	
	for (i = 0; (uint8_t)pgm_read_byte(&FlashLoc[i]); i++)
		LCDsendChar((uint8_t)pgm_read_byte(&FlashLoc[i]));
}

//defines char symbol in CGRAM
/*
const uint8_t backslash[] PROGMEM= 
{
0b00000000,//back slash
0b00010000,
0b00001000,
0b00000100,
0b00000010,
0b00000001,
0b00000000,
0b00000000
};

LCDdefinechar(backslash,0);
*/

void LCDdefinechar(const uint8_t *pc,uint8_t char_code)
{
	uint8_t a, pcc;
	uint16_t i;

	a = (char_code<<3) | 0x40;

	for (i=0; i<8; i++)
	{
		pcc = pgm_read_byte(&pc[i]);
		LCDsendCommand(a++);
		LCDsendChar(pcc);
	}
}

void LCDshiftLeft(uint8_t n)	//Scrol n of characters Right
{
	uint8_t i;
	for (i = 0; i < n; i++)
		LCDsendCommand(0x1E);
}

void LCDshiftRight(uint8_t n)	//Scrol n of characters Left
{
	uint8_t i;
	for (i = 0; i < n;i++)
		LCDsendCommand(0x18);
}

void LCDcursorOn(void) //displays LCD cursor
{
	LCDsendCommand(0x0E);
}

void LCDcursorOnBlink(void)	//displays LCD blinking cursor
{
	LCDsendCommand(0x0F);
}

char LCDcursorOFF(void)	//turns OFF cursor
{
	LCDsendCommand(0x0C);
}

void LCDblank(void)		//blanks LCD
{
	LCDsendCommand(0x08);
}

void LCDvisible(void)		//Shows LCD
{
	LCDsendCommand(0x0C);
}

void LCDcursorLeft(uint8_t n)	//Moves cursor by n poisitions left
{
	uint8_t i;
	for (i = 0; i < n; i++)
		LCDsendCommand(0x10);
}

void LCDcursorRight(uint8_t n)	//Moves cursor by n poisitions left
{
	uint8_t i;
	for (i = 0; i < n; i++)
		LCDsendCommand(0x14);
}


//==========================================================================//

void LCD_strcpy (char *dest, const char *source)
{
	char len = strlen(source);

	if (len >= LCD_STR_SIZE)
		len = LCD_STR_SIZE;

	char i;
	for (i = 0; i < len; i++)
		dest[i] = source[i];

	for (;i < LCD_STR_SIZE; i++)
		dest[i] = 0;
}

void PrintStr (char *str)
{
	int i;
	for (i = 0; i < LCD_RUNNING_STRING_DELAY; i++)
		_delay_ms(100);

	LCD_strcpy(LCD_str1, LCD_str2);

	int len = strlen(str);

	if (len > LCD_STR_SIZE)
		len = LCD_STR_SIZE;

	LCD_strcpy(LCD_str2, str);

	LCDclr();   
	
	LCDGotoXY(1,0);	
	LCDstring(LCD_str1,strlen(LCD_str1));	
	LCDGotoXY(1,1);
	LCDstring(LCD_str2, strlen(LCD_str2));
}

//=====================================================================//
